@@ -18,14 +18,7 @@ from __future__ import print_function
 
 import os
 
-from . import get_keras_submodule
-
-backend = get_keras_submodule('backend')
-engine = get_keras_submodule('engine')
-layers = get_keras_submodule('layers')
-models = get_keras_submodule('models')
-keras_utils = get_keras_submodule('utils')
-
+from . import get_submodules_from_kwargs
 from . import imagenet_utils
 from .imagenet_utils import decode_predictions
 from .imagenet_utils import _obtain_input_shape
@@ -53,6 +46,11 @@ DENSENET201_WEIGHT_PATH_NO_TOP = (
     BASE_WEIGTHS_PATH +
     'densenet201_weights_tf_dim_ordering_tf_kernels_notop.h5')
 
+backend = None
+layers = None
+models = None
+keras_utils = None
+
 
 def dense_block(x, blocks, name):
     """A dense block.
@@ -128,7 +126,8 @@ def DenseNet(blocks,
              input_tensor=None,
              input_shape=None,
              pooling=None,
-             classes=1000):
+             classes=1000,
+             **kwargs):
     """Instantiates the DenseNet architecture.
 
     Optionally loads weights pre-trained on ImageNet.
@@ -147,9 +146,11 @@ def DenseNet(blocks,
             to use as image input for the model.
         input_shape: optional shape tuple, only to be specified
             if `include_top` is False (otherwise the input shape
-            has to be `(224, 224, 3)` (with `channels_last` data format)
-            or `(3, 224, 224)` (with `channels_first` data format).
-            It should have exactly 3 inputs channels.
+            has to be `(224, 224, 3)` (with `'channels_last'` data format)
+            or `(3, 224, 224)` (with `'channels_first'` data format).
+            It should have exactly 3 inputs channels,
+            and width and height should be no smaller than 32.
+            E.g. `(200, 200, 3)` would be one valid value.
         pooling: optional pooling mode for feature extraction
             when `include_top` is `False`.
             - `None` means that the output of the model will be
@@ -172,6 +173,9 @@ def DenseNet(blocks,
         ValueError: in case of invalid argument for `weights`,
             or invalid input shape.
     """
+    global backend, layers, models, keras_utils
+    backend, layers, models, keras_utils = get_submodules_from_kwargs(kwargs)
+
     if not (weights in {'imagenet', None} or os.path.exists(weights)):
         raise ValueError('The `weights` argument should be either '
                          '`None` (random initialization), `imagenet` '
@@ -185,7 +189,7 @@ def DenseNet(blocks,
     # Determine proper input shape
     input_shape = _obtain_input_shape(input_shape,
                                       default_size=224,
-                                      min_size=221,
+                                      min_size=32,
                                       data_format=backend.image_data_format(),
                                       require_flatten=include_top,
                                       weights=weights)
@@ -218,6 +222,7 @@ def DenseNet(blocks,
 
     x = layers.BatchNormalization(
         axis=bn_axis, epsilon=1.001e-5, name='bn')(x)
+    x = layers.Activation('relu', name='relu')(x)
 
     if include_top:
         x = layers.GlobalAveragePooling2D(name='avg_pool')(x)
@@ -231,11 +236,7 @@ def DenseNet(blocks,
     # Ensure that the model takes into account
     # any potential predecessors of `input_tensor`.
     if input_tensor is not None:
-        if hasattr(keras_utils, 'get_source_inputs'):
-            get_source_inputs = keras_utils.get_source_inputs
-        else:
-            get_source_inputs = engine.get_source_inputs
-        inputs = get_source_inputs(input_tensor)
+        inputs = keras_utils.get_source_inputs(input_tensor)
     else:
         inputs = img_input
 
@@ -301,11 +302,13 @@ def DenseNet121(include_top=True,
                 input_tensor=None,
                 input_shape=None,
                 pooling=None,
-                classes=1000):
+                classes=1000,
+                **kwargs):
     return DenseNet([6, 12, 24, 16],
                     include_top, weights,
                     input_tensor, input_shape,
-                    pooling, classes)
+                    pooling, classes,
+                    **kwargs)
 
 
 def DenseNet169(include_top=True,
@@ -313,11 +316,13 @@ def DenseNet169(include_top=True,
                 input_tensor=None,
                 input_shape=None,
                 pooling=None,
-                classes=1000):
+                classes=1000,
+                **kwargs):
     return DenseNet([6, 12, 32, 32],
                     include_top, weights,
                     input_tensor, input_shape,
-                    pooling, classes)
+                    pooling, classes,
+                    **kwargs)
 
 
 def DenseNet201(include_top=True,
@@ -325,14 +330,16 @@ def DenseNet201(include_top=True,
                 input_tensor=None,
                 input_shape=None,
                 pooling=None,
-                classes=1000):
+                classes=1000,
+                **kwargs):
     return DenseNet([6, 12, 48, 32],
                     include_top, weights,
                     input_tensor, input_shape,
-                    pooling, classes)
+                    pooling, classes,
+                    **kwargs)
 
 
-def preprocess_input(x, data_format=None):
+def preprocess_input(x, data_format=None, **kwargs):
     """Preprocesses a numpy array encoding a batch of images.
 
     # Arguments
@@ -342,7 +349,8 @@ def preprocess_input(x, data_format=None):
     # Returns
         Preprocessed array.
     """
-    return imagenet_utils.preprocess_input(x, data_format, mode='torch')
+    return imagenet_utils.preprocess_input(x, data_format,
+                                           mode='torch', **kwargs)
 
 
 setattr(DenseNet121, '__doc__', DenseNet.__doc__)
